<button type="button" class="btn btn-default" aria-label="Left Align" onclick=DeleteTempEntity("{{ record.pk }}", 'personPlace')>
  <span class="glyphicon glyphicon-remove" aria-hidden="true"></span>
</button>