{% load apis_helpers %}
<a class='reledit' onclick=GetFormAjax("{{ record|content_type}}Form","{% if record.relation_pk %}{{record.relation_pk}}{% else %}{{ record.pk}}{% endif %}")><svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-edit-2"><polygon points="16 3 21 8 8 21 3 21 3 16 16 3"></polygon></svg></a>
