# Generated by Django 2.2.13 on 2020-07-22 12:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apis_entities', '0003_auto_20200216_1152'),
    ]

    operations = [
        migrations.AlterField(
            model_name='person',
            name='gender',
            field=models.CharField(blank=True, choices=[('female', 'female'), ('male', 'male'), ('third gender', 'third gender')], max_length=15, null=True),
        ),
    ]
